// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.loghandler;

public class IdGenerator
{
  private static final ThreadLocal threadNumberLocal = new ThreadLocal();
  private static final ThreadLocal stackNumberLocal = new ThreadLocal();
  private static volatile int threadSequenceNumber = 0;
  private static volatile long globalSequenceNumber = 0;

  public static synchronized long getThreadId()
  {
    Long id = (Long) threadNumberLocal.get();
    if (id == null) {
      id = new Long(threadSequenceNumber++);
      threadNumberLocal.set(id);
    }
    return id.longValue();
  }

  public static long putOnStack()
  {
    StackNumber id = getStackNumber();
    id.increaseStackNumber();
    return id.getStackNumber();
  }
  
  public static long removeFromStack()
  {
    StackNumber id = getStackNumber();
    id.decreaseStackNumber();
    return id.getStackNumber();
  }
  
  private static StackNumber getStackNumber()
  {
    StackNumber stackNumber = (StackNumber) stackNumberLocal.get();
    if (stackNumber == null) {
      stackNumber = new StackNumber();
      stackNumberLocal.set(stackNumber);
    }
    return stackNumber;
  }
  
  public static synchronized long getNextSequenceNumber()
  {
    return globalSequenceNumber++;
  }
  
  private static class StackNumber
  {
    private long currentStackNumber = 0;

    public long getStackNumber()
    {
      return currentStackNumber;
    }
    
    public void increaseStackNumber()
    {
      currentStackNumber++;
    }
    
    public void decreaseStackNumber()
    {
      currentStackNumber--;
    }
  }
}
