package performancemonster.accumulatedcallgraph.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import performancemonster.accumulatedcallgraph.MethodNode;

public class MethodNodeImpl implements MethodNode
{
  private List<MethodNode> children = new ArrayList<MethodNode>();
  private final Set<performancemonster.stretchedcallgraph.MethodNode> accumulatedNodes = new HashSet<performancemonster.stretchedcallgraph.MethodNode>();
  private final MethodNode parent;

  public MethodNodeImpl(MethodNode parent)
  {
    this.parent = parent;
  }
  
  public void addElement(performancemonster.stretchedcallgraph.MethodNode node)
  {
    accumulatedNodes.add(node);
  }

  public void replaceElement(
      performancemonster.stretchedcallgraph.MethodNode node)
  {
    if (!accumulatedNodes.contains(node)) {
      throw new IllegalArgumentException("The node '" + node
          + "' is not already in this node");
    }
    accumulatedNodes.remove(node);
    accumulatedNodes.add(node);
  }
  
  public void addChild(MethodNode node)
  {
    children.add(node);
  }
  
  public MethodNode getParent()
  {
    return parent;
  }
  
  public List<MethodNode> getChildren()
  {
    return children;
  }
  
  public Set<Long> getElementIds()
  {
    Set<Long> ids = new HashSet<Long>();
    for (performancemonster.stretchedcallgraph.MethodNode node : accumulatedNodes) {
      ids.add(node.getElementId());
    }
    return ids;
  }
  
  public long getTotalTime()
  {
    long totalTime = 0;
    for (performancemonster.stretchedcallgraph.MethodNode node : accumulatedNodes) {
      if (node.isEnded()) {
        totalTime += node.getTotalTime();
      }
    }
    return totalTime;
  }

  public long getTimeInMethod()
  {
    long timeInMethod = 0;
    for (performancemonster.stretchedcallgraph.MethodNode node : accumulatedNodes) {
      if (node.isThisAndSublevelEnded()) {
        timeInMethod += node.getTimeInMethod();
      }
    }
    return timeInMethod;
  }

  public long getNumberOfCalls()
  {
    return accumulatedNodes.size();
  }

  public String getName()
  {
    return accumulatedNodes.iterator().next().getName();
  }
  
  public boolean isEnded()
  {
    for (performancemonster.stretchedcallgraph.MethodNode node : accumulatedNodes) {
      if (!node.isEnded()) {
        return false;
      }
    }
    return true;
  }

  @Override
  public String toString()
  {
    return "MethodNodeImpl " + getName() + " (called " + getNumberOfCalls()
        + " times) " + getTotalTime() + " / " + getTimeInMethod();
  }
}
