// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.accumulatedcallgraph;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.util.Collections;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.tree.TreePath;

import org.jdesktop.swingx.JXTreeTable;

import performancemonster.accumulatedcallgraph.Analyzer;
import performancemonster.accumulatedcallgraph.CallGraphChangeListener;
import performancemonster.accumulatedcallgraph.CallGraphError;
import performancemonster.graph.GraphModel;
import performancemonster.presentation.accumulatedcallgraph.impl.CallGraphTreeModel;
import performancemonster.presentation.accumulatedcallgraph.impl.ChangeListener;
import performancemonster.presentation.accumulatedcallgraph.impl.ErrorList;
import performancemonster.presentation.accumulatedcallgraph.impl.MethodNode;
import performancemonster.presentation.accumulatedcallgraph.impl.Node;
import performancemonster.presentation.accumulatedcallgraph.impl.SummaryNode;
import performancemonster.presentation.helpers.LayoutManagerHandler;
import performancemonster.presentation.helpers.LayoutManagerHandler.Direction;

public class AccumulatedCallGraphPanel extends JPanel
{
  private final GraphModel model;
  
  public AccumulatedCallGraphPanel(GraphModel model)
  {
    this.model = model;
    performancemonster.stretchedcallgraph.Analyzer stretchedAnalyzer = new performancemonster.stretchedcallgraph.Analyzer(
        model);
    Analyzer analyzer = new Analyzer(stretchedAnalyzer);
    setLayout(new BorderLayout());
    
    SummaryNode summary = new SummaryNode();
    final CallGraphTreeModel treeModel = new CallGraphTreeModel(summary);
    final JXTreeTable table = new JXTreeTable(treeModel);
    analyzer.addChangeListener(new ChangeListener(summary, treeModel, table));
    DefaultTableCellRenderer stringRenderer = new DefaultTableCellRenderer();
    stringRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
    table.setDefaultRenderer(String.class, stringRenderer);

    table.getSelectionModel().addListSelectionListener(
        new ListSelectionListener()
        {
          public void valueChanged(ListSelectionEvent e)
          {
            int selectedRow = table.getSelectedRow();
            if (selectedRow != -1) {
              TreePath treePath = table.getPathForRow(selectedRow);
              Node node = (Node) treePath.getLastPathComponent();
              if (node instanceof MethodNode) {
                MethodNode methodNode = (MethodNode) node;
                AccumulatedCallGraphPanel.this.model.setElementsInFocus(methodNode
                    .getElementIds());
                return;
              }
            }
            Set<Long> emptySet = Collections.emptySet();
            AccumulatedCallGraphPanel.this.model.setElementsInFocus(emptySet);
          }
        });
    
    add(new JScrollPane(table), BorderLayout.CENTER);
    add(createErrorPanel(analyzer), BorderLayout.SOUTH);
  }
  
  private JPanel createErrorPanel(Analyzer analyzer)
  {
    final JFrame errorFrame = new ErrorList(analyzer);
    
    LayoutManagerHandler handler = new LayoutManagerHandler(
        Direction.HORIZONTAL);
    Action showErrorsAction = new AbstractAction("Show errors")
    {
      public void actionPerformed(ActionEvent e)
      {
        errorFrame.setVisible(true);
      }
    };
    final JLabel numberOfErrors = new JLabel("No errors");
    handler.addComponent(numberOfErrors, true, 1.0);
    handler.addComponent(new JButton(showErrorsAction), true, 1.0);

    analyzer.addChangeListener(new CallGraphChangeListener() {
      int errorNumber = 0;

      public void methodAdded(performancemonster.accumulatedcallgraph.MethodNode node)
      {
        // Intentionally left blank
      }

      public void methodUpdated(performancemonster.accumulatedcallgraph.MethodNode node)
      {
        // Intentionally left blank
      }
      public void errorAdded(CallGraphError error)
      {
        errorNumber++;
        numberOfErrors.setText("Errors: " + errorNumber);
      }

      public void elementsCleared()
      {
        errorNumber = 0;
        numberOfErrors.setText("No errors");
      }

      public void methodsSwitched(performancemonster.accumulatedcallgraph.MethodNode goingDown, performancemonster.accumulatedcallgraph.MethodNode goingUp)
      {
        // Intentionally left blank
      }});
    
    return handler.getPanel();
  }
}
