// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.accumulatedcallgraph.impl;

import javax.swing.tree.TreeNode;

import org.jdesktop.swingx.treetable.AbstractTreeTableModel;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;

public final class CallGraphTreeModel extends DefaultTreeTableModel
{
  // Names of the columns.
  private static final String[] cNames = { "Name", "Calls", "Total time",
      "In method" };

  // Types of the columns.
  private static final Class[] cTypes = {
      AbstractTreeTableModel.hierarchicalColumnClass, Long.class, String.class,
      String.class };

  public CallGraphTreeModel(TreeNode root)
  {
    super(root);
  }

  public int getColumnCount()
  {
    return cNames.length;
  }

  public String getColumnName(int column)
  {
    return cNames[column];
  }

  public Class getColumnClass(int column)
  {
    return cTypes[column];
  }

  public boolean isCellEditable(Object node, int column)
  {
    return column == 0;
  }

  public Object getValueAt(Object object, int column)
  {
    Node node = (Node) object;
    switch (column) {
    case 0:
      return node.getName();
    case 1:
      return node.getNumberOfCalls();
    case 2:
      return node.getTotalTime();
    case 3:
      return node.getTimeInMethod();
    }
    return null;
  }
}
