package performancemonster.presentation.accumulatedcallgraph.impl;

import java.util.HashMap;
import java.util.Map;

import javax.swing.tree.TreePath;

import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;

import performancemonster.accumulatedcallgraph.CallGraphChangeListener;
import performancemonster.accumulatedcallgraph.CallGraphError;
import performancemonster.accumulatedcallgraph.MethodNode;

public class ChangeListener implements CallGraphChangeListener
{
  private final SummaryNode summaryNode;
  private final DefaultTreeTableModel model;
  private final JXTreeTable table;
  private final Map<MethodNode, performancemonster.presentation.accumulatedcallgraph.impl.MethodNode> methodNodes;
  
  public ChangeListener(SummaryNode summaryNode, DefaultTreeTableModel model,
      JXTreeTable table)
  {
    this.summaryNode = summaryNode;
    this.model = model;
    this.table = table;
    methodNodes = new HashMap<MethodNode, performancemonster.presentation.accumulatedcallgraph.impl.MethodNode>();
  }
  
  public void methodAdded(MethodNode node)
  {
    Node guiParent = getParent(node);
    performancemonster.presentation.accumulatedcallgraph.impl.MethodNode guiNode = new performancemonster.presentation.accumulatedcallgraph.impl.MethodNode(
        guiParent, node);
    methodNodes.put(node, guiNode);
    guiParent.addChild(guiNode);
    int[] indices = {guiParent.getChildCount() - 1};
    model.nodesWereInserted(guiParent, indices);
    model.nodesChanged(guiParent, indices);
    table.expandPath(null);
    TreePath pathToRoot = new TreePath(model.getPathToRoot(guiParent));
    table.expandPath(pathToRoot);
  }

  public void methodUpdated(MethodNode node)
  {
    performancemonster.presentation.accumulatedcallgraph.impl.MethodNode guiNode = methodNodes
        .get(node);
    int[] indices = { guiNode.getParent().getIndex(guiNode) };
    model.nodesChanged(guiNode.getParent(), indices);
  }

  public void methodsSwitched(MethodNode goingDown, MethodNode goingUp)
  {
    performancemonster.presentation.accumulatedcallgraph.impl.MethodNode guiNodeGoingDown = methodNodes.get(goingDown);
    performancemonster.presentation.accumulatedcallgraph.impl.MethodNode guiNodeGoingUp = methodNodes.get(goingUp);
    Node guiParent = guiNodeGoingDown.getParent();
    guiParent.switchNodes(guiNodeGoingDown, guiNodeGoingUp);
    model.nodeStructureChanged(guiParent);
    for (Node node : guiParent.getChildren()) {
      TreePath pathToRoot = new TreePath(model.getPathToRoot(node));
      table.expandPath(pathToRoot);
    }
  }

  public void errorAdded(CallGraphError error)
  {
    // intentionally left blank
  }

  public void elementsCleared()
  {
    methodNodes.clear();
    summaryNode.clearChildren();
    model.nodeStructureChanged(summaryNode);
  }
  
  private Node getParent(MethodNode node)
  {
    MethodNode parent = node.getParent();
    if (parent != null) {
      return methodNodes.get(parent);
    } else {
      return summaryNode;
    }
  }
}
