// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.accumulatedcallgraph.impl;

import java.util.Set;

public final class MethodNode extends Node
{
  private final Node parent;
  private final performancemonster.accumulatedcallgraph.MethodNode node;

  public MethodNode(Node parent, performancemonster.accumulatedcallgraph.MethodNode node)
  {
    this.parent = parent;
    this.node = node;
    for (performancemonster.accumulatedcallgraph.MethodNode method : node.getChildren()) {
      children.add(new MethodNode(this, method));
    }
  }

  @Override
  public Node getParent()
  {
    return parent;
  }

  @Override
  public String toString()
  {
    return getName();
  }

  @Override
  public String getTotalTime()
  {
    return node.getTotalTime() + " ms" + (node.isEnded() ? "" : "+");
  }

  @Override
  public String getTimeInMethod()
  {
    return node.getTimeInMethod() + " ms";
  }

  @Override
  public String getName()
  {
    return node.getName();
  }

  @Override
  public long getNumberOfCalls()
  {
    return node.getNumberOfCalls();
  }
  
  public Set<Long> getElementIds()
  {
    return node.getElementIds();
  }
}
