// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.accumulatedcallgraph.impl;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.swing.tree.TreeNode;

import performancemonster.helpers.EnumerationAdapter;

public abstract class Node implements TreeNode
{
  protected final List<Node> children = new ArrayList<Node>();

  public abstract TreeNode getParent();
  public abstract String getName();
  public abstract long getNumberOfCalls();
  public abstract String getTotalTime();
  public abstract String getTimeInMethod();
  
  public void switchNodes(Node goingDown, Node goingUp)
  {
    int firstIndex = getIndex(goingDown);
    children.set(firstIndex, goingUp);
    children.set(firstIndex + 1, goingDown);
  }
  
  public List<Node> getChildren()
  {
    return children;
  }
  
  public TreeNode getChildAt(int childIndex)
  {
    return children.get(childIndex);
  }

  public int getChildCount()
  {
    return children.size();
  }

  public int getIndex(TreeNode node)
  {
    return children.indexOf(node);
  }

  public boolean getAllowsChildren()
  {
    return true;
  }

  public boolean isLeaf()
  {
    return children.size() == 0;
  }

  public Enumeration children()
  {
    return new EnumerationAdapter(children.iterator());
  }
  
  public void addChild(Node child)
  {
    children.add(child);
  }
}
