// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.stretchedcallgraph;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.tree.TreePath;

import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;

import performancemonster.graph.GraphModel;
import performancemonster.presentation.helpers.LayoutManagerHandler;
import performancemonster.presentation.helpers.LayoutManagerHandler.Direction;
import performancemonster.presentation.stretchedcallgraph.impl.ChangeListener;
import performancemonster.presentation.stretchedcallgraph.impl.ErrorList;
import performancemonster.presentation.stretchedcallgraph.impl.MethodNode;
import performancemonster.presentation.stretchedcallgraph.impl.Node;
import performancemonster.presentation.stretchedcallgraph.impl.StretchedCallGraphTreeModel;
import performancemonster.presentation.stretchedcallgraph.impl.SummaryNode;
import performancemonster.stretchedcallgraph.Analyzer;
import performancemonster.stretchedcallgraph.CallGraphChangeListener;
import performancemonster.stretchedcallgraph.CallGraphError;
import performancemonster.stretchedcallgraph.ThreadNode;

public class StretchedCallGraphPanel extends JPanel
{
  private final GraphModel model;
  private final DefaultTreeTableModel treeTableModel;
  
  public StretchedCallGraphPanel(GraphModel model)
  {
    this.model = model;
    setLayout(new BorderLayout());

    Analyzer analyzer = new Analyzer(model);
    Collection<ThreadNode> threads = analyzer.getThreads();
    SummaryNode summaryNode = new SummaryNode(threads);
    treeTableModel = new StretchedCallGraphTreeModel(summaryNode);
    final JXTreeTable table = new JXTreeTable(treeTableModel);
    analyzer.addChangeListener(new ChangeListener(summaryNode, treeTableModel,
        table));
    DefaultTableCellRenderer stringRenderer = new DefaultTableCellRenderer();
    stringRenderer.setHorizontalAlignment(SwingConstants.RIGHT);
    table.setDefaultRenderer(String.class, stringRenderer);

    table.getSelectionModel().addListSelectionListener(
        new ListSelectionListener()
        {
          public void valueChanged(ListSelectionEvent e)
          {
            int selectedRow = table.getSelectedRow();
            if (selectedRow != -1) {
              TreePath treePath = table.getPathForRow(selectedRow);
              Node node = (Node) treePath.getLastPathComponent();
              if (node instanceof MethodNode) {
                MethodNode methodNode = (MethodNode) node;
                Set<Long> elementInFocus = new HashSet<Long>();
                elementInFocus.add(methodNode.getElementId());
                StretchedCallGraphPanel.this.model
                    .setElementsInFocus(elementInFocus);
                return;
              }
            }
            Set<Long> emptySet = Collections.emptySet();
            StretchedCallGraphPanel.this.model.setElementsInFocus(emptySet);
          }
        });
    
    add(new JScrollPane(table), BorderLayout.CENTER);
    add(createErrorPanel(analyzer), BorderLayout.SOUTH);
  }
  
  private JPanel createErrorPanel(Analyzer analyzer)
  {
    final JFrame errorFrame = new ErrorList(analyzer);
    
    LayoutManagerHandler handler = new LayoutManagerHandler(
        Direction.HORIZONTAL);
    Action showErrorsAction = new AbstractAction("Show errors")
    {
      public void actionPerformed(ActionEvent e)
      {
        errorFrame.setVisible(true);
      }
    };
    final JLabel numberOfErrors = new JLabel("No errors");
    handler.addComponent(numberOfErrors, true, 1.0);
    handler.addComponent(new JButton(showErrorsAction), true, 1.0);

    analyzer.addChangeListener(new CallGraphChangeListener() {
      int errorNumber = 0;
      
      public void threadAdded(ThreadNode node)
      {
        // Intentionally left blank
      }

      public void methodAdded(performancemonster.stretchedcallgraph.MethodNode node)
      {
        // Intentionally left blank
      }

      public void methodUpdated(performancemonster.stretchedcallgraph.MethodNode node)
      {
        // Intentionally left blank
      }

      public void errorAdded(CallGraphError error)
      {
        errorNumber++;
        numberOfErrors.setText("Errors: " + errorNumber);
      }

      public void elementsCleared()
      {
        errorNumber = 0;
        numberOfErrors.setText("No errors");
      }});
    
    return handler.getPanel();
  }
}
