package performancemonster.presentation.stretchedcallgraph.impl;

import java.util.HashMap;
import java.util.Map;

import javax.swing.tree.TreePath;

import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;

import performancemonster.stretchedcallgraph.CallGraphChangeListener;
import performancemonster.stretchedcallgraph.CallGraphError;
import performancemonster.stretchedcallgraph.MethodNode;
import performancemonster.stretchedcallgraph.ThreadNode;

public class ChangeListener implements CallGraphChangeListener
{
  private final SummaryNode summaryNode;
  private final Map<Long, performancemonster.presentation.stretchedcallgraph.impl.MethodNode> methodNodes;
  private final Map<Long, performancemonster.presentation.stretchedcallgraph.impl.ThreadNode> threadNodes;
  private final DefaultTreeTableModel model;
  private final JXTreeTable table;
  
  public ChangeListener(SummaryNode summaryNode, DefaultTreeTableModel model, JXTreeTable table)
  {
    this.summaryNode = summaryNode;
    this.model = model;
    this.table = table;
    methodNodes = new HashMap<Long, performancemonster.presentation.stretchedcallgraph.impl.MethodNode>();
    threadNodes = new HashMap<Long, performancemonster.presentation.stretchedcallgraph.impl.ThreadNode>();
  }
  
  
  public void threadAdded(ThreadNode node)
  {
    performancemonster.presentation.stretchedcallgraph.impl.ThreadNode guiNode = new performancemonster.presentation.stretchedcallgraph.impl.ThreadNode(
        summaryNode, node);
    threadNodes.put(guiNode.getId(), guiNode);
    summaryNode.addChild(guiNode);
    int[] indices = {summaryNode.getChildCount() - 1};
    model.nodesWereInserted(summaryNode, indices);
    model.nodesChanged(summaryNode, indices);
    TreePath pathToRoot = new TreePath(model.getPathToRoot(summaryNode));
    table.expandPath(pathToRoot);
  }

  public void methodAdded(performancemonster.stretchedcallgraph.MethodNode node)
  {
    Node parentNode = getParent(node);
    performancemonster.presentation.stretchedcallgraph.impl.MethodNode guiNode = new performancemonster.presentation.stretchedcallgraph.impl.MethodNode(parentNode, node);
    methodNodes.put(guiNode.getElementId(), guiNode);
    parentNode.addChild(guiNode);
    int[] indices = {parentNode.getChildCount() - 1};
    model.nodesWereInserted(parentNode, indices);
    model.nodesChanged(parentNode, indices);
    table.expandPath(null);
    TreePath pathToRoot = new TreePath(model.getPathToRoot(parentNode));
    table.expandPath(pathToRoot);
  }

  public void methodUpdated(
      performancemonster.stretchedcallgraph.MethodNode node)
  {
    performancemonster.presentation.stretchedcallgraph.impl.MethodNode guiNode = methodNodes
        .get(node.getElementId());
    guiNode.changeNode(node);
    int[] indices = {guiNode.getParent().getIndex(guiNode)};
    model.nodesChanged(guiNode.getParent(), indices);
  }
  
  public void errorAdded(CallGraphError error)
  {
    // Intentionally left blank
  }

  public void elementsCleared()
  {
    methodNodes.clear();
    summaryNode.clearChildren();
    model.nodeStructureChanged(summaryNode);
  }
  
  private Node getParent(MethodNode node)
  {
    performancemonster.stretchedcallgraph.Node parent = node.getParent();
    if (parent instanceof MethodNode) {
      MethodNode methodNode = (MethodNode) parent;
      return methodNodes.get(methodNode.getElementId());
    } else if (parent instanceof ThreadNode) {
      ThreadNode threadNode = (ThreadNode) parent;
      return threadNodes.get(threadNode.getThreadId());
    }
    throw new IllegalArgumentException("Parent of node '" + node
        + "' is of unsupported type: '" + parent + "'");
  }
}
