// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.stretchedcallgraph.impl;

import javax.swing.tree.TreeNode;

public final class MethodNode extends Node
{
  private final Node parent;
  private performancemonster.stretchedcallgraph.MethodNode node;

  public MethodNode(Node parent,
      performancemonster.stretchedcallgraph.MethodNode node)
  {
    this.parent = parent;
    this.node = node;
  }

  @Override
  public TreeNode getParent()
  {
    return parent;
  }

  @Override
  public String toString()
  {
    return getName();
  }

  @Override
  public String getTotalTime()
  {
    if (node.isEnded()) {
      return node.getTotalTime() + " ms";
    } else {
      return "(Unbounded)";
    }
  }

  @Override
  public String getTimeInMethod()
  {
    if (node.isThisAndSublevelEnded()) {
      return node.getTimeInMethod() + " ms";
    } else {
      return "(Unbounded)";
    }
  }

  @Override
  public String getName()
  {
    return node.getName();
  }
  
  @Override
  public long getId()
  {
    return node.getElementId();
  }
  
  public void changeNode(performancemonster.stretchedcallgraph.MethodNode newNode)
  {
    node = newNode;
  }

  public long getElementId()
  {
    return node.getElementId();
  }
}
