package performancemonster.stretchedcallgraph;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import performancemonster.graph.GraphElement;
import performancemonster.stretchedcallgraph.impl.MethodNodeImpl;

public class ThreadNode implements Node
{
  private final long threadId;
  private final List<MethodNode> children;
  
  public ThreadNode(long threadId)
  {
    this.threadId = threadId;
    this.children = new ArrayList<MethodNode>();
  }

  public Node addChild(GraphElement element)
  {
    MethodNode node = new MethodNodeImpl(this, element);
    children.add(node);
    return node;
  }
  
  public Node getParent()
  {
    throw new UnsupportedOperationException(
        "getParent() not applicable to ThreadNode");
  }
  
  public List<MethodNode> getChildren()
  {
    return Collections.unmodifiableList(children);
  }
  
  public String toString()
  {
    StringBuilder string = new StringBuilder();
    string.append("Thread ");
    string.append(threadId);
    string.append("(");
    for (Node node : children) {
      string.append(node.toString());
    }
    string.append(")");
    return string.toString();
  }

  public String getName()
  {
    return "Thread " + threadId;
  }

  public long getTotalTime()
  {
    long totalTime = 0;
    for (Node node : children) {
      totalTime += node.getTotalTime();
    }
    return totalTime;
  }

  public long getTimeInMethod()
  {
    return getTotalTime();
  }

  public long getThreadId()
  {
    return threadId;
  }

  public boolean isEnded()
  {
    return areChildrenEnded();
  }

  public boolean areChildrenEnded()
  {
    for (Node node : children) {
      if (!node.isEnded()) {
        return false;
      }
    }
    return true;
  }

  @Override
  public int hashCode()
  {
    return (int) getThreadId();
  }
  
  @Override
  public boolean equals(Object other)
  {
    if (!(other instanceof ThreadNode)) {
      return false;
    }
    return ((ThreadNode) other).getThreadId() == getThreadId();
  }
}
