package performancemonster.stretchedcallgraph.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import performancemonster.graph.GraphElement;
import performancemonster.stretchedcallgraph.MethodNode;
import performancemonster.stretchedcallgraph.Node;

public class MethodNodeImpl implements MethodNode
{
  private List<MethodNode> children = new ArrayList<MethodNode>();
  private final Node parent;
  private GraphElement element;

  public MethodNodeImpl(Node parent, GraphElement element)
  {
    if (parent == null) {
      throw new IllegalArgumentException("Parent is null");
    }
    if (element == null) {
      throw new IllegalArgumentException("Element is null");
    }
    this.parent = parent;
    this.element = element;
  }
  
  public Node addChild(GraphElement element)
  {
    MethodNodeImpl node = new MethodNodeImpl(this, element);
    children.add(node);
    return node;
  }
  
  public void updateInformation(GraphElement newElement)
  {
    element = newElement;
  }
  
  public Node getParent()
  {
    return parent;
  }
  
  public List<MethodNode> getChildren()
  {
    return Collections.unmodifiableList(children);
  }
  
  public long getElementId()
  {
    return element.getId();
  }
  
  public String toString()
  {
    StringBuilder string = new StringBuilder();
    string.append("Method ");
    string.append(element.getMessage());
    string.append(": ");
    string.append(element.getEnd() - element.getBeginning());
    string.append(" (");
    for (Node node : children) {
      string.append(node.toString());
    }
    string.append(")");
    return string.toString();
  }

  public boolean isEnded()
  {
    return element.isEnded();
  }
  
  public boolean isThisAndSublevelEnded()
  {
    if (!element.isEnded()) {
      return false;
    }
    for (MethodNode child : children) {
      if (!child.isEnded()) {
        return false;
      }
    }
    return true;
  }
  
  public long getEnd()
  {
    return element.getEnd();
  }
  
  public long getTotalTime()
  {
    if (!isEnded()) {
      throw new IllegalStateException(
          "getTotalTime() makes no sense when isEnded() is false");
    }
    return element.getEnd() - element.getBeginning();
  }

  public long getTimeInMethod()
  {
    if (!isEnded()) {
      throw new IllegalStateException(
          "getTimeInMethod() makes no sense when isEnded() is false");
    }
    long timeInSubMethods = 0;
    for (Node node : children) {
      timeInSubMethods += node.getTotalTime();
    }
    return getTotalTime() - timeInSubMethods;
  }

  public String getName()
  {
    return element.getMessage();
  }
  
  @Override
  public int hashCode()
  {
    return getName().hashCode();
  }
  
  @Override
  public boolean equals(Object other)
  {
    if (!(other instanceof MethodNodeImpl)) {
      return false;
    }
    return ((MethodNodeImpl) other).element == element;
  }
}
